function setUpReferences()
 	la = love.audio
	le = love.event
	lf = love.filesystem
	lg = love.graphics
	li = love.image
	lj = love.joystick
	lk = love.keyboard
	lma = love.math
	lm = love.mouse
	lp = love.physics
	lso = love.sound
	lsy = love.system
	lth = love.thread
	lti = love.timer
	lw = love.window
end

function love.load(arg)
  setUpReferences()
  time = 0
  timeLimitU = 0.99
  timeLimitD = 0.01
  toggle = 1
  
  points = 60 -- amount of points
  
  --spiral
  spiralRadius = 300
  spirals = 8
  spiralVariation = 2
  spiralX, spiralY = lg.getWidth()/2, lg.getHeight()/2
  spiralDecrement = (spiralRadius-40)/(points)
  
  --Wave
  waveLimitX = 800
  waveDistance = 200
  waveOffY = 100
  waveCenterY = lg.getHeight()/2
  
  colors = {{math.random(0, 255), math.random(0, 255), math.random(0, 255), math.random(2, 10)/10},
    {math.random(0, 255), math.random(0, 255), math.random(0, 255), math.random(5, 20)}}
  radius = {32}
  
  curveTypes = {"wave", "random", "spiral"}
  currType = 1
  
  generatePoints(curveTypes[currType])
  
  math.randomseed(os.time())
end

function love.draw()
  lg.setColor(255, 0, 0)
  local text = "Type: " .. curveTypes[currType] .. "\nTime: " .. time
  lg.print(text)
  
  lg.setLineWidth(4)
  local point, total = evaluateBezier(time, lines)
  lg.setBlendMode("additive")
  for i = 2, #total do
    lg.setColor(colors[1])
    drawMiddlePoints(time, total[i], radius[1])
    lg.setColor(colors[2])
    drawMiddleLines(time, total[i])
  end
end

function love.update(dt)
  if time > timeLimitU then
    time = timeLimitU
    toggle = -1
    --timeLimitD = math.random(0, timeLimitU*1000)/1000
    colors[1] = {math.random(0, 255), math.random(0, 255), math.random(0, 255), math.random(10, 20)/10}
    radius[1] = math.random(8, 48)
  end
  if time < timeLimitD then
    time = timeLimitD
    toggle = 1
    --timeLimitU = math.random(timeLimitD*1000, 1000)/1000
    colors[2] = {math.random(0, 255), math.random(0, 255), math.random(0, 255), math.random(5, 20)}
    radius[1] = math.random(8, 48)
  end
  
  local speed = 1-math.abs((time - 0.5)*2)
  local mul = 1+speed*80
  
  time = time + ((dt*toggle)/(points))*mul
end

function love.keypressed(key, unicode)
  
  
  if key == "p" or key == "o" then
    if key == "o" then
      currType = currType + 1
      if currType > #curveTypes then
        currType = currType - #curveTypes
      end
    end
    
    lines = {}
    generatePoints(curveTypes[currType])
  end
end
  
  

function pointInLine(t, line)
  local lx = line[3]-line[1]
  local ly = line[4]-line[2]
  lx = lx*t+line[1]
  ly = ly*t+line[2]
  
  return {lx, ly}
end

function evaluateBezier(t, points, total)
  total = total or {}
  total[#total+1] = {}
  
  for i = 1, #points do
    table.insert(total[#total], points[i])
  end
  --print(points)
  if #points > 2 then
    local nP = {}
    for i = 1, #points-2, 2 do
      nP[i], nP[i+1] = unpack(pointInLine(t, {points[i], points[i+1], points[i+2], points[i+3]}))
    end
    return evaluateBezier(t, nP, total)
  else
    return points, total
  end
end

function drawMiddlePoints(t, points, size)
  for i = 1, #points, 2 do
    local x, y = points[i], points[i+1]--unpack(pointInLine(t, {points[i], points[i+1], points[i+2], points[i+3]}))
    lg.circle("fill", x, y, size)
  end
end

function drawMiddleLines(t, points)
  for i = 1, #points-2, 2 do
    lg.line(points[i], points[i+1], points[i+2], points[i+3])
  end
end

function generatePoints(type)
  lines = {}
  
  if type == "random" then
  
  --random
      for i = 1, points do
        table.insert(lines, math.random(80, lg.getWidth()-80))
        table.insert(lines, math.random(80, lg.getHeight()-80))
      end
  
  
  elseif type == "spiral" then
  
  --spiral
      local totalSpiral = (spiralRadius-40)/spirals
      for i = spiralRadius, 40, -spiralDecrement do
        local angle = ((i+math.random(-spiralVariation, spiralVariation))*spirals)%360
        table.insert(lines, spiralX+math.sin(math.rad(angle))*i)
        table.insert(lines, spiralY+math.cos(math.rad(angle))*i)
      end
  
  
  elseif type == "wave" then
  
  --wave
      local waveLength = waveLimitX/points
      local top = 1
      for i = 1, points do
        local x = waveLength*i
        local y = waveCenterY + (waveDistance+math.random(-waveOffY, waveOffY))*top
        table.insert(lines, x)
        table.insert(lines, y)
        top = top * -1
      end
      
      local x, y = 0, 0
      for i = 1, #lines, 2 do
        lines[i], lines[i+1] = lines[i]+x, lines[i+1]+y
      end
      
  end
end